summaryrefslogtreecommitdiff
path: root/hurd/hurdsig.c
diff options
context:
space:
mode:
authorRoland McGrath <roland@gnu.org>1999-12-03 05:01:23 +0000
committerRoland McGrath <roland@gnu.org>1999-12-03 05:01:23 +0000
commit72e1a75071d8c4b269e03c583f84c9a130434062 (patch)
treece119ac1bf830309de109a401f799608390694b3 /hurd/hurdsig.c
parent63bf605304e93d6eda7aa11b43ace572a3bced30 (diff)
1999-11-18 Roland McGrath <roland@baalperazim.frob.com>
* hurd/hurdsig.c (_hurdsig_init): If __hurd_threadvar_stack_mask is nonzero, use cthread_fork to create the signal thread. * hurd/msgportdemux.c (_hurd_msgport_receive): Initialize _hurd_msgport_thread here (to self). * sysdeps/mach/hurd/fork.c (__fork): When __hurd_sigthread_stack_end is zero, instead compute child signal thread's starting SP from parent signal thread's current SP and the threadvar_stack variables. * hurd/Versions (GLIBC_2.1.3): Add cthread_fork, cthread_detach. These are now referenced weakly by _hurdsig_init. * hurd/report-wait.c (_S_msg_report_wait): Fix typo: &_hurd_itimer_thread not &_hurd_msgport_thread. 1999-10-01 Roland McGrath <roland@baalperazim.frob.com> * hurd/hurdfchdir.c (_hurd_change_directory_port_from_fd): Rewrite without HURD_DPORT_USE to clean up warnings. * hurd/dtable.c (get_dtable_port): Likewise. * hurd/hurdioctl.c (rectty_dtable): Renamed to install_ctty. (install_ctty): Do the changing of the cttyid port cell here, inside the critical section while we holding the dtable lock. (_hurd_setcttyid, tiocsctty, tiocnotty): Use that instead of changing the port cell and calling rectty_dtable. (_hurd_locked_install_cttyid): New function, split out of install_ctty. (install_ctty): Use it inside a critical section, with the lock held. * sysdeps/mach/hurd/setsid.c (__setsid): Use _hurd_locked_install_cttyid to effect the cttyid and dtable changes after proc_setsid, having held the dtable lock throughout. * hurd/dtable.c (ctty_new_pgrp): With the dtable lock held, check the cttyid port for null and bail out early if so. The dtable lock serializes us after any cttyid change and its associated dtable update.
Diffstat (limited to 'hurd/hurdsig.c')
-rw-r--r--hurd/hurdsig.c55
1 files changed, 37 insertions, 18 deletions
diff --git a/hurd/hurdsig.c b/hurd/hurdsig.c
index 2c9625b474..47f5fbf0a4 100644
--- a/hurd/hurdsig.c
+++ b/hurd/hurdsig.c
@@ -1227,28 +1227,47 @@ _hurdsig_init (const int *intarray, size_t intarraysize)
/* Start the signal thread listening on the message port. */
- err = __thread_create (__mach_task_self (), &_hurd_msgport_thread);
- assert_perror (err);
+ if (__hurd_threadvar_stack_mask == 0)
+ {
+ err = __thread_create (__mach_task_self (), &_hurd_msgport_thread);
+ assert_perror (err);
- stacksize = __vm_page_size * 8; /* Small stack for signal thread. */
- err = __mach_setup_thread (__mach_task_self (), _hurd_msgport_thread,
- _hurd_msgport_receive,
- (vm_address_t *) &__hurd_sigthread_stack_base,
- &stacksize);
- assert_perror (err);
+ stacksize = ~__hurd_threadvar_stack_mask + 1;
+ stacksize = __vm_page_size * 8; /* Small stack for signal thread. */
+ err = __mach_setup_thread (__mach_task_self (), _hurd_msgport_thread,
+ _hurd_msgport_receive,
+ (vm_address_t *) &__hurd_sigthread_stack_base,
+ &stacksize);
+ assert_perror (err);
- __hurd_sigthread_stack_end = __hurd_sigthread_stack_base + stacksize;
- __hurd_sigthread_variables =
- malloc (__hurd_threadvar_max * sizeof (unsigned long int));
- if (__hurd_sigthread_variables == NULL)
- __libc_fatal ("hurd: Can't allocate thread variables for signal thread\n");
+ __hurd_sigthread_stack_end = __hurd_sigthread_stack_base + stacksize;
+ __hurd_sigthread_variables =
+ malloc (__hurd_threadvar_max * sizeof (unsigned long int));
+ if (__hurd_sigthread_variables == NULL)
+ __libc_fatal ("hurd: Can't allocate threadvars for signal thread\n");
- /* Reinitialize the MiG support routines so they will use a per-thread
- variable for the cached reply port. */
- __mig_init ((void *) __hurd_sigthread_stack_base);
+ /* Reinitialize the MiG support routines so they will use a per-thread
+ variable for the cached reply port. */
+ __mig_init ((void *) __hurd_sigthread_stack_base);
- err = __thread_resume (_hurd_msgport_thread);
- assert_perror (err);
+ err = __thread_resume (_hurd_msgport_thread);
+ assert_perror (err);
+ }
+ else
+ {
+ /* When cthreads is being used, we need to make the signal thread a
+ proper cthread. Otherwise it cannot use mutex_lock et al, which
+ will be the cthreads versions. Various of the message port RPC
+ handlers need to take locks, so we need to be able to call into
+ cthreads code and meet its assumptions about how our thread and
+ its stack are arranged. Since cthreads puts it there anyway,
+ we'll let the signal thread's per-thread variables be found as for
+ any normal cthread, and just leave the magic __hurd_sigthread_*
+ values all zero so they'll be ignored. */
+#pragma weak cthread_fork
+#pragma weak cthread_detach
+ cthread_detach (cthread_fork ((cthread_fn_t) &_hurd_msgport_receive, 0));
+ }
/* Receive exceptions on the signal port. */
__task_set_special_port (__mach_task_self (),